#ifndef __FixedDistanceInit__h
#define __FixedDistanceInit__h

/*
 * Author : SV
 * Date   : 
 *   version 1 - 2013-04-23
 *   version 2 - 2013-11-05 
 */

#include <minimizers/solution/solution.h>
#include <coord3d.h>

/*
 *
 * Random initialisation according to the size of recepteor and ligand
 * the random position are on sphere. 
 * The radius is at least the sum of the radius of the molecules.
 * 
 */
class FixedDistanceInit {
public:
  /*
   * Constructor using the receptor and the ligand
   *
   * @param _rng random generator
   * @param _rec the receptor
   * @param _lig the ligand
   * @param _coef the distance will be betwen [radius_receptor + radius_ligand , radius_receptor + coef * radius_ligand ] 
   */
  FixedDistanceInit(std::mt19937 & _rng, PTools::AttractRigidbody & _rec, PTools::AttractRigidbody & _lig, double _coef = 1.01) : rng(_rng), uniform(0,1), coef(_coef) {
    recRadius = _rec.Radius();
    ligRadius = _lig.Radius();

    center = _rec.FindCenter();
    
    if (coef <= 1) 
      coef = 1.01;
  }

  /**
   * Initialisation of the solution randomly on a sphere
   *
   * @param _solution the coordinates for the position of the ligand
   **/
  void apply(Solution & _solution) {
    // first 3 dimensions for rotation
    // last  3 dimensions for translation coordinate

    // TODO : change this definition PI
    double PI = 3.14159265358979323846;

    // rotation
    _solution.coord[0] = 2 * PI * uniform(rng) ; // [0, 2 PI]
    _solution.coord[1] = 2 * PI * uniform(rng) ; // [0, 2 PI]
    _solution.coord[2] = 2 * PI * uniform(rng) ; // [0, 2 PI]

    // translation
    double distance = recRadius + ( 1  + (coef - 1) * uniform(rng) ) * ligRadius ;
    double theta = 2 * PI * uniform(rng) ;      // [0, 2 PI]
    double phi   = 2 * PI * uniform(rng) - PI ; // [-PI, PI]
    
    _solution.coord[3] = center.x + cos(theta) * cos(phi) * distance ;
    _solution.coord[4] = center.y + sin(theta) * cos(phi) * distance ;
    _solution.coord[5] = center.z + sin(phi) * distance ;
  }

protected:
  // radius of the receptor
  double recRadius;

  // radius of the ligand
  double ligRadius;

  // center of the receptor
  PTools::Coord3D center;

  // coefficient applied to the distance
  double coef ;

private:
  // random generator
  std::mt19937 & rng;

  // random number: uniform distribution 
  std::uniform_real_distribution<> uniform;
  //std::normal_distribution<double> normal_dist(mean, stddeviation);

};

#endif
